/*
** Module   : NativeUnsignedInt32.java
** Abstract : descriptor for an unsigned 32-bit integer 
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 GES 20131212 First version.
** 002 GES 20140212 Non-scalar instances must emit as a sign-extended value of the platform's
**                  word-size instead of the actual size of the instance.
*/

package com.goldencode.p2j.library;

import com.goldencode.p2j.util.*;

/**
 * A descriptor for an unsigned 32-bit integer.
 */
public class NativeUnsignedInt32 
extends NativeInteger
{
   /**
    * Default constructor.
    */
   public NativeUnsignedInt32()
   {
      super();
      type = LibraryManager.TYPE_UNSIGNED_INT32;
   }
   
   /**
    * Report the size (in bytes) of the native memory needed to store this instance.
    * <p>
    * This should never be called on the server, only on the client side.
    *
    * @return   The number of bytes needed.
    */
   public int size()
   {
      return isInArray() ? 4 : LibraryManager.ptrSize();
   }
   
   /**
    * Worker method to read the primitive value from the first index position of the given
    * memory buffer.
    *
    * @param    ptr
    *           The memory buffer to reference.
    */
   protected void readNative(memptr ptr)
   {
      int64 num = ptr.getUnsignedLong(1);
      
      if (!num.isUnknown())
      {
         value = num.longValue();
      }
   }
   
   /**
    * Worker method to write the primitive value to the first index position of the given
    * memory buffer.
    *
    * @param    ptr
    *           The memory buffer to reference.
    */
   protected void writeNative(memptr ptr)
   {
      if (isInArray())
      {
         ptr.setUnsignedLong(value, 1);
      }
      else
      {
         writeNativeWordSize(ptr);
      }
   }
}
